==> vdr-mediamvp-0.1.1.tgz: mediamvp-0.1.1/menuapp-vdr.c <==

/*
 *   MediaMVP Server
 *
 *   (C) 2003 Dominic Morris
 *
 *   $Id: menuapp-vdr.c,v 1.3 2004/05/07 20:52:09 dom Exp $
 *   $Date: 2004/05/07 20:52:09 $
 *
 *
 *   Contains vdr specific functionality for the menuapp
 */


#include "mediamvp.h"

#include "misc.h"
#if 0
#include <vdr/i18n.h>
#include <vdr/eit.h>
#include <vdr/recording.h>
#endif


static int    channels_colour_keys(menu_t *menu, void *imenuapp, int key, int sel);
static int    schedule_colour_keys(menu_t *menu, void *imenuapp, int key, int sel);
static int    vdrrecs_colour_keys(menu_t *menu, void *imenuapp, int key, int sel);

/***********************
 *
 *   Handling of the Channel Menus
 *
 ***********************/
static int menuapp_channels(menu_t *menu,void *imenuapp, int sel)
{
    char           buf[512]; 
    char           buf2[512]; 
    char           buf3[20]; 
    menuapp_t     *menuapp = (menuapp_t *)imenuapp;
    dirlist_t     *dirlist;
    filelist_t    *list = NULL;
    int            list_num = 0;
    menu_t        *child;
    cChannel      *chan;

    /* Now, create a new menu for us */
    dirlist = new_dirlist(menuapp);
    child = new_menu(menuapp->dongle,menuapp->render,menuapp->program,menu,dirlist,0);
    menu_set_param_clearup(child,delete_dirlist);
    menu_set_dialcode_callback(child, dial_entry);
    menu_set_title(child,tr("Channels")); 

    for (chan = Channels.First(); chan != NULL; chan = Channels.Next(chan) ) {
        if ( chan->GroupSep() && !*chan->Name())
            continue;

        if ( chan->GroupSep() ) {

        } else {
            snprintf(buf,sizeof(buf),"% 3d %s",chan->Number(),chan->Name());
            snprintf(buf2,sizeof(buf2),"%s%s",VDRURL,chan->GetChannelID().ToString());
            /* Make the dial code the channel number */
            snprintf(buf3,sizeof(buf3),"%d",chan->Number());
            list = filelist_add(list,&list_num,buf,buf2,strdup(buf3),FILE_URL);
            menu_add(child,buf,process_media);
        }
    }

    /* Now we'll set up dirlist */
    dirlist->title = NULL;    /* We don't need it since there's no subdirs */
    dirlist->path  = NULL;    /* We don't need it sinec there's no subdirs */
    dirlist->filelist_num = list_num;
    dirlist->filelist = list;

    menu_set_colour_actions(child,"Back",NULL,NULL,tr("Switch"),channels_colour_keys);

    menu_display(child);
    return 1;
}



static int channels_colour_keys(menu_t *menu, void *imenuapp, int key, int sel)
{
    dirlist_t   *dir = (dirlist_t *)imenuapp;
    filelist_t  *entry = &dir->filelist[sel];
    menuapp_t   *menuapp = dir->menuapp;

    switch ( key ) {
    case keyRed:
        return 0 ;
    case keyBlue:
        if ( entry == NULL ) {   /* Nothing there.. */
            return 1;
        }
        if ( menuapp->current_media == NULL ) {        
            menuapp->current_media = strdup(entry->url);
            menuapp->paused = FALSE;
            dongle_send_play(menuapp->dongle,entry->url);
        } else {
            next_media_clear(menuapp);
            next_media_push(menuapp,entry->url);       
            dongle_send_message(menuapp->dongle,RDC_STOP);
        }
        program_register_keys(menuapp->program,REGISTER_MEDIA,media_keys,menuapp);
        program_register_ack(menuapp->program,REGISTER_MEDIA,media_ack,menuapp);
        return 1;   
    default:
        break;
    }

    return 1;     /* Carry on displaying this menu */
}

/***********************
 *
 *   Handling of the Schedule Menus
 *
 ***********************/
static int CompareEventChannel(const void *p1, const void *p2)
{
#if VDRVERSNUM < 10300
  return (int)( (*(const cEventInfo **)p1)->GetChannelNumber() - (*(const cEventInfo **)p2)->GetChannelNumber());
#else
  return (int)( (*(const cEvent **)p1)->ChannelNumber() - (*(const cEvent **)p2)->ChannelNumber());
#endif
}

static int schedule_menuapp(menu_t *menu, void *imenuapp, int sel)
{
#if VDRVERSNUM < 10300
    cMutexLock MutexLock;
    const cSchedules *Schedules = cSIProcessor::Schedules(MutexLock);
    const cEventInfo **pArray = NULL;
#else
    cSchedulesLock MutexLock;
    const cSchedules *Schedules = cSchedules::Schedules(MutexLock);
    const cEvent **pArray = NULL;
#endif
    menuapp_t     *menuapp = (menuapp_t *)imenuapp;
    dirlist_t     *dirlist;
    filelist_t    *list = NULL;
    int            list_num = 0;
    menu_t        *child;
    const cSchedule *Schedule;
    int num = 0;

    if ( Schedules != NULL ) {
        Schedule = Schedules->First();

        /* If it ain't broke, why fix it? - Stolen from vdr */
        while ( Schedule ) {
#if VDRVERSNUM < 10300
            pArray = (const cEventInfo **)realloc(pArray, (num + 1) * sizeof(cEventInfo *));
#else
            pArray = (const cEvent **)realloc(pArray, (num + 1) * sizeof(cEvent *));
#endif
            
            pArray[num] = menuapp->Now ? Schedule->GetPresentEvent() : Schedule->GetFollowingEvent();
            
            if (pArray[num]) {
#if VDRVERSNUM < 10300
                cChannel *channel = Channels.GetByChannelID(pArray[num]->GetChannelID(), true);
#else
                cChannel *channel = Channels.GetByChannelID(pArray[num]->ChannelID(), true);
#endif
                if (channel) {
                    pArray[num]->SetChannelNumber(channel->Number());
                    num++;
                }
            }
            Schedule = (const cSchedule *)Schedules->Next(Schedule);
        }
#if VDRVERSNUM < 10300
        qsort(pArray, num, sizeof(cEventInfo *), CompareEventChannel);
#else
        qsort(pArray, num, sizeof(cEvent *), CompareEventChannel);
#endif
    }
        
    /* Now, create a new menu for us */
    dirlist = new_dirlist(menuapp);
    child = new_menu(menuapp->dongle,menuapp->render,menuapp->program,menu,dirlist,0);
    menu_set_param_clearup(child,delete_dirlist);
    menu_set_dialcode_callback(child, dial_entry);

    menu_set_title(child,menuapp->Now ? tr("What's on now?") : tr("What's on next?"));
   
    for ( int i = 0; i < num; i++ ) {
        char       buf[1024];
        char       buf2[1024];
        char       buf3[20];
#if VDRVERSNUM < 10300
        const cEventInfo *eventInfo = pArray[i];
        cChannel *channel = Channels.GetByNumber(eventInfo->GetChannelNumber());
#else
        const cEvent *eventInfo = pArray[i];
        cChannel *channel = Channels.GetByNumber(eventInfo->ChannelNumber());
#endif


#if VDRVERSNUM < 10300
        snprintf(buf,sizeof(buf),"%d\t%.*s\t%.*s\t%s", 
                 pArray[i]->GetChannelNumber(), 6,  channel ? channel->Name() : "???",
                 5, eventInfo->GetTimeString(), eventInfo->GetTitle());
#else
        snprintf(buf,sizeof(buf),"%d\t%.*s\t%.*s\t%s", 
                 pArray[i]->ChannelNumber(), 6,  channel ? channel->Name() : "???",
                 5, eventInfo->GetTimeString(), eventInfo->Title());
#endif
        snprintf(buf2,sizeof(buf2),"%s%s",VDRURL, channel ? channel->GetChannelID().ToString() : "???");

        snprintf(buf3,sizeof(buf3),"%d",channel ? channel->Number() : 0);


        list = filelist_add(list,&list_num,buf,buf2,channel ? strdup(buf3) : NULL,FILE_URL);
        menu_add(child,buf,process_media);       
    }
    if ( pArray != NULL ) {
        free(pArray);
    }

    /* Now we'll set up dirlist */
    dirlist->title = NULL;    /* We don't need it since there's no subdirs */
    dirlist->path  = NULL;    /* We don't need it sinec there's no subdirs */
    dirlist->filelist_num = list_num;
    dirlist->filelist = list;


    menu_set_colour_actions(child,tr("Back") /* tr("Record") */, menuapp->Now ? tr("Next") : tr("Now"), NULL /* tr("Schedule") */, tr("Switch"),schedule_colour_keys);

    menu_display(child);

    return 1;
}


static int schedule_colour_keys(menu_t *menu, void *imenuapp, int key, int sel)
{
    dirlist_t   *dir = (dirlist_t *)imenuapp;
    filelist_t  *entry = &dir->filelist[sel];
    menuapp_t   *menuapp = dir->menuapp;
    menu_t      *parent;

    switch ( key ) {
    case keyRed:
        return 0 ;
    case keyGreen:
        menuapp->Now ^= 1;
        parent = menu_parent(menu);
        schedule_menuapp(parent,menuapp,0);
        return 0;
    case keyYellow:
        /* Display the schedule for a channel */   
        break;   
    case keyBlue:     /* Switch to channel */
        if ( entry == NULL ) {   /* Nothing there.. */
            return 1;
        }
        if ( menuapp->current_media == NULL ) {        
            menuapp->current_media = strdup(entry->url);
            menuapp->paused = FALSE;
            dongle_send_play(menuapp->dongle,entry->url);
        } else {
            next_media_clear(menuapp);
            next_media_push(menuapp,entry->url);       
            dongle_send_message(menuapp->dongle,RDC_STOP);
        }
        program_register_keys(menuapp->program,REGISTER_MEDIA,media_keys,menuapp);
        program_register_ack(menuapp->program,REGISTER_MEDIA,media_ack,menuapp);
        break;
    }   
    return 1;
}

/***********************
 *
 *   Handling of the vdrrecs Menus
 *
 ***********************/
static int menuapp_vdrrecs(menu_t *menu,void *imenuapp, int sel)
{
    menuapp_t      *menuapp = (menuapp_t *)imenuapp;
    menu_t         *child;
    dirlist_t      *dirlist;
    filelist_t     *list = NULL;
    int             num  = 0;
    char            buf[1024];
    char            buf2[1024];
    cRecordings     Recordings;

    /* Define a menu */
    dirlist = new_dirlist(menuapp);
    child = new_menu(menuapp->dongle,menuapp->render,menuapp->program,menu,dirlist,0);
    menu_set_param_clearup(child,delete_dirlist);

    menu_set_title(child,tr("Video Recordings"));   
    menu_set_dialcode_callback(child, dial_entry);

    /* Load recordings up from the VDR core */
    bool recordings = Recordings.Load();

    /* Thanks Sascha! - code taken from streamdev-server */
    if ( recordings ) {
        cRecording *recording = Recordings.First();
        while ( recording ) {
            
            snprintf(buf,sizeof(buf),"%s",recording->Title(' ',true));
            snprintf(buf2,sizeof(buf2),"%s%s",VDRRECURL,buf);
            
            list = filelist_add(list,&num,buf,buf2,NULL,FILE_URL);
            menu_add(child,buf,process_media);
          
            recording = Recordings.Next(recording);
        }
    }
   
    /* Now we'll set up dirlist */
    dirlist->title = NULL;    /* We don't need it since there's no subdirs */
    dirlist->path  = NULL;    /* We don't need it sinec there's no subdirs */
    dirlist->filelist_num = num;
    dirlist->filelist = list;
   
    menu_set_colour_actions(child,tr("Back"),NULL,NULL,NULL,vdrrecs_colour_keys);

    menu_display(child);
    return 1;  
}


static int vdrrecs_colour_keys(menu_t *menu, void *imenuapp, int key, int sel)
{
    switch ( key ) {
    case keyRed:
        return 0;   
    default:
        break;
    }

    return 1;     /* Carry on displaying this menu */
}


/* Deal with Chan+/Chan- */
int  media_keys_vdr(menuapp_t *menuapp,int code)
{
    int            streamtype = dongle_get_type(menuapp->dongle);
    
    switch ( code ) {
    case keyChanUp:
    case keyChanDn:
        if (  ( (streamtype & (MEDIA_LIVE | MEDIA_MPEG) ) == (MEDIA_LIVE | MEDIA_MPEG) ) ) {
            if ( menuapp->current_media && strncmp(menuapp->current_media,VDRURL,strlen(VDRURL)) == 0  ) {
                char           buf[2048];
                const char    *newchannel = ChannelSkip(menuapp->current_media + strlen(VDRURL),
                                                        code == keyChanUp ? 1 : -1 );
                if ( newchannel ) {
                    if ( menuapp->current_media ) {        
                        free(menuapp->current_media);
		    }
                    snprintf(buf,sizeof(buf),"%s%s",VDRURL,newchannel);
                    menuapp->current_media = strdup(buf);
                    menuapp->paused = FALSE;
                    dongle_send_play(menuapp->dongle,buf);
                }
            }
            return 1;  /* We've done all we can.. */
        } else {
            return 0;   /* Carry on with menu */
        }
    }

    return -1;         /* Carry on with normal key functions */
}


/* Scan a file in the format used by the mp3 plugin */
static filelist_t *plugin_file_scan(char *filename, int *scanned)
{
    char         buf[FILENAME_MAX+1];
    filelist_t  *list = NULL;
    int          count = 0;
    char        *ptr,*endptr;
    struct stat  sb;
    FILE        *fp;
    int          flags;

    *scanned = 0;


    if ( stat(filename,&sb) < 0 || ( fp = fopen(filename,"r") ) == NULL ) {
        return NULL;
    }
  
    while ( !feof(fp) && fgets(buf,sizeof(buf),fp) ) {
        if ( (ptr = strchr(buf,';') ) == NULL ) {
            continue;
        }
        *ptr = 0;
        ptr++;

        if ( (endptr = strchr(ptr,';') ) == NULL && 
             (endptr = strchr(ptr,'\n') ) == NULL &&
             (endptr = strchr(ptr,'\r') ) == NULL ) {
            continue;
        }
        *endptr = 0;

        if ( stat(buf,&sb) < 0 ) {
            continue;
        }

        if ( S_ISDIR(sb.st_mode) ) {
            flags = FILE_DIR;
        } else {
            flags = 0;
        }

        list = filelist_add(list,&count,ptr,buf,NULL,flags);       
    }

    *scanned = count;
    return list;    
}

==> vdr-mediamvp-0.1.2.tgz: mediamvp-0.1.2/menuapp-vdr.c <==

/*
 *   MediaMVP Server
 *
 *   (C) 2003 Dominic Morris
 *
 *   $Id: menuapp-vdr.c,v 1.5 2004/05/22 15:11:18 dom Exp $
 *   $Date: 2004/05/22 15:11:18 $
 *
 *
 *   Contains vdr specific functionality for the menuapp
 */


#include "mediamvp.h"

#include "misc.h"
#if 0
#include <vdr/i18n.h>
#include <vdr/eit.h>
#include <vdr/recording.h>
#endif


static int    channels_colour_keys(menu_t *menu, void *imenuapp, int key, int sel);
static int    schedule_colour_keys(menu_t *menu, void *imenuapp, int key, int sel);
static int    vdrrecs_colour_keys(menu_t *menu, void *imenuapp, int key, int sel);

/***********************
 *
 *   Handling of the Channel Menus
 *
 ***********************/
static int menuapp_channels(menu_t *menu,void *imenuapp, int sel)
{
    char           buf[512]; 
    char           buf2[512]; 
    char           buf3[20]; 
    menuapp_t     *menuapp = (menuapp_t *)imenuapp;
    dirlist_t     *dirlist;
    filelist_t    *list = NULL;
    int            list_num = 0;
    menu_t        *child;
    cChannel      *chan;

    /* Now, create a new menu for us */
    dirlist = new_dirlist(menuapp);
    child = new_menu(menuapp->dongle,menuapp->render,menuapp->program,menu,dirlist,0);
    menu_set_param_clearup(child,delete_dirlist);
    menu_set_dialcode_callback(child, dial_entry);
    menu_set_title(child,tr("Channels")); 

    for (chan = Channels.First(); chan != NULL; chan = Channels.Next(chan) ) {
        if ( chan->GroupSep() && !*chan->Name())
            continue;

        if ( chan->GroupSep() ) {

        } else {
            snprintf(buf,sizeof(buf),"% 3d %s",chan->Number(),chan->Name());
            snprintf(buf2,sizeof(buf2),"%s%s",VDRURL,chan->GetChannelID().ToString());
            /* Make the dial code the channel number */
            snprintf(buf3,sizeof(buf3),"%d",chan->Number());
            list = filelist_add(list,&list_num,buf,buf2,strdup(buf3),FILE_URL);
            menu_add(child,buf,process_media);
        }
    }

    /* Now we'll set up dirlist */
    dirlist->title = NULL;    /* We don't need it since there's no subdirs */
    dirlist->path  = NULL;    /* We don't need it sinec there's no subdirs */
    dirlist->filelist_num = list_num;
    dirlist->filelist = list;

    menu_set_colour_actions(child,"Back",NULL,NULL,tr("Switch"),channels_colour_keys);

    menu_display(child);
    return 1;
}



static int channels_colour_keys(menu_t *menu, void *imenuapp, int key, int sel)
{
    dirlist_t   *dir = (dirlist_t *)imenuapp;
    filelist_t  *entry = &dir->filelist[sel];
    menuapp_t   *menuapp = dir->menuapp;

    switch ( key ) {
    case keyRed:
        return 0 ;
    case keyBlue:
        if ( entry == NULL ) {   /* Nothing there.. */
            return 1;
        }
        if ( menuapp->current_media == NULL ) {        
            menuapp->current_media = strdup(entry->url);
            menuapp->paused = FALSE;
            dongle_send_play(menuapp->dongle,entry->url);
        } else {
            next_media_clear(menuapp);
            next_media_push(menuapp,entry->url);       
            dongle_send_message(menuapp->dongle,RDC_STOP);
        }
        program_register_keys(menuapp->program,REGISTER_MEDIA,media_keys,menuapp);
        program_register_ack(menuapp->program,REGISTER_MEDIA,media_ack,menuapp);
        return 1;   
    default:
        break;
    }

    return 1;     /* Carry on displaying this menu */
}

/***********************
 *
 *   Handling of the Schedule Menus
 *
 ***********************/
#if VDRVERSNUM >= 10305
static int schedule_menuapp(menu_t *menu, void *imenuapp, int sel)
{
    menuapp_t     *menuapp = (menuapp_t *)imenuapp;
    dirlist_t     *dirlist;
    filelist_t    *list = NULL;
    int            list_num = 0;
    menu_t        *child;
    cSchedulesLock MutexLock;
    const cSchedules *Schedules = cSchedules::Schedules(MutexLock);

    /* Now, create a new menu for us */
    dirlist = new_dirlist(menuapp);
    child = new_menu(menuapp->dongle,menuapp->render,menuapp->program,menu,dirlist,0);
    menu_set_param_clearup(child,delete_dirlist);
    menu_set_dialcode_callback(child, dial_entry);

    menu_set_title(child,menuapp->Now ? tr("What's on now?") : tr("What's on next?"));
   

    for (cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel)) {
        if (!Channel->GroupSep()) {
            const cSchedule *Schedule = Schedules->GetSchedule(Channel->GetChannelID());
            if (Schedule) {
                const cEvent *Event = menuapp->Now ? Schedule->GetPresentEvent() : Schedule->GetFollowingEvent();
                if (Event) {
                    char       buf[1024];
                    char       buf2[1024];
                    char       buf3[20];
                    snprintf(buf,sizeof(buf),"%d\t%.*s\t%.*s\t%s", Channel->Number(), 6, Channel->Name(), 5, Event->GetTimeString(), Event->Title());
    
                    snprintf(buf2,sizeof(buf2),"%s%s",VDRURL, Channel->GetChannelID().ToString());

                    snprintf(buf3,sizeof(buf3),"%d",Channel->Number());


                    list = filelist_add(list,&list_num,buf,buf2,strdup(buf3),FILE_URL);
                    menu_add(child,buf,process_media);       
  
                }
            }
        }
    }
    /* Now we'll set up dirlist */
    dirlist->title = NULL;    /* We don't need it since there's no subdirs */
    dirlist->path  = NULL;    /* We don't need it sinec there's no subdirs */
    dirlist->filelist_num = list_num;
    dirlist->filelist = list;


    menu_set_colour_actions(child,tr("Back") /* tr("Record") */, menuapp->Now ? tr("Next") : tr("Now"), NULL /* tr("Schedule") */, tr("Switch"),schedule_colour_keys);

    menu_display(child);

    return 1;
}

#else
static int CompareEventChannel(const void *p1, const void *p2)
{
#if VDRVERSNUM < 10300
  return (int)( (*(const cEventInfo **)p1)->GetChannelNumber() - (*(const cEventInfo **)p2)->GetChannelNumber());
#else
  return (int)( (*(const cEvent **)p1)->ChannelNumber() - (*(const cEvent **)p2)->ChannelNumber());
#endif
}

static int schedule_menuapp(menu_t *menu, void *imenuapp, int sel)
{
#if VDRVERSNUM < 10300
    cMutexLock MutexLock;
    const cSchedules *Schedules = cSIProcessor::Schedules(MutexLock);
    const cEventInfo **pArray = NULL;
#else
    cSchedulesLock MutexLock;
    const cSchedules *Schedules = cSchedules::Schedules(MutexLock);
    const cEvent **pArray = NULL;
#endif
    menuapp_t     *menuapp = (menuapp_t *)imenuapp;
    dirlist_t     *dirlist;
    filelist_t    *list = NULL;
    int            list_num = 0;
    menu_t        *child;
    const cSchedule *Schedule;
    int num = 0;

    if ( Schedules != NULL ) {
        Schedule = Schedules->First();

        /* If it ain't broke, why fix it? - Stolen from vdr */
        while ( Schedule ) {
#if VDRVERSNUM < 10300
            pArray = (const cEventInfo **)realloc(pArray, (num + 1) * sizeof(cEventInfo *));
#else
            pArray = (const cEvent **)realloc(pArray, (num + 1) * sizeof(cEvent *));
#endif
            
            pArray[num] = menuapp->Now ? Schedule->GetPresentEvent() : Schedule->GetFollowingEvent();
            
            if (pArray[num]) {
#if VDRVERSNUM < 10300
                cChannel *channel = Channels.GetByChannelID(pArray[num]->GetChannelID(), true);
#else
                cChannel *channel = Channels.GetByChannelID(pArray[num]->ChannelID(), true);
#endif
                if (channel) {
                    pArray[num]->SetChannelNumber(channel->Number());
                    num++;
                }
            }
            Schedule = (const cSchedule *)Schedules->Next(Schedule);
        }
#if VDRVERSNUM < 10300
        qsort(pArray, num, sizeof(cEventInfo *), CompareEventChannel);
#else
        qsort(pArray, num, sizeof(cEvent *), CompareEventChannel);
#endif
    }
        
    /* Now, create a new menu for us */
    dirlist = new_dirlist(menuapp);
    child = new_menu(menuapp->dongle,menuapp->render,menuapp->program,menu,dirlist,0);
    menu_set_param_clearup(child,delete_dirlist);
    menu_set_dialcode_callback(child, dial_entry);

    menu_set_title(child,menuapp->Now ? tr("What's on now?") : tr("What's on next?"));
   
    for ( int i = 0; i < num; i++ ) {
        char       buf[1024];
        char       buf2[1024];
        char       buf3[20];
#if VDRVERSNUM < 10300
        const cEventInfo *eventInfo = pArray[i];
        cChannel *channel = Channels.GetByNumber(eventInfo->GetChannelNumber());
#else
        const cEvent *eventInfo = pArray[i];
        cChannel *channel = Channels.GetByNumber(eventInfo->ChannelNumber());
#endif


#if VDRVERSNUM < 10300
        snprintf(buf,sizeof(buf),"%d\t%.*s\t%.*s\t%s", 
                 pArray[i]->GetChannelNumber(), 6,  channel ? channel->Name() : "???",
                 5, eventInfo->GetTimeString(), eventInfo->GetTitle());
#else
        snprintf(buf,sizeof(buf),"%d\t%.*s\t%.*s\t%s", 
                 pArray[i]->ChannelNumber(), 6,  channel ? channel->Name() : "???",
                 5, eventInfo->GetTimeString(), eventInfo->Title());
#endif
        snprintf(buf2,sizeof(buf2),"%s%s",VDRURL, channel ? channel->GetChannelID().ToString() : "???");

        snprintf(buf3,sizeof(buf3),"%d",channel ? channel->Number() : 0);


        list = filelist_add(list,&list_num,buf,buf2,channel ? strdup(buf3) : NULL,FILE_URL);
        menu_add(child,buf,process_media);       
    }
    if ( pArray != NULL ) {
        free(pArray);
    }

    /* Now we'll set up dirlist */
    dirlist->title = NULL;    /* We don't need it since there's no subdirs */
    dirlist->path  = NULL;    /* We don't need it sinec there's no subdirs */
    dirlist->filelist_num = list_num;
    dirlist->filelist = list;


    menu_set_colour_actions(child,tr("Back") /* tr("Record") */, menuapp->Now ? tr("Next") : tr("Now"), NULL /* tr("Schedule") */, tr("Switch"),schedule_colour_keys);

    menu_display(child);

    return 1;
}
#endif


static int schedule_colour_keys(menu_t *menu, void *imenuapp, int key, int sel)
{
    dirlist_t   *dir = (dirlist_t *)imenuapp;
    filelist_t  *entry = &dir->filelist[sel];
    menuapp_t   *menuapp = dir->menuapp;
    menu_t      *parent;

    switch ( key ) {
    case keyRed:
        return 0 ;
    case keyGreen:
        menuapp->Now ^= 1;
        parent = menu_parent(menu);
        schedule_menuapp(parent,menuapp,0);
        return 0;
    case keyYellow:
        /* Display the schedule for a channel */   
        break;   
    case keyBlue:     /* Switch to channel */
        if ( entry == NULL ) {   /* Nothing there.. */
            return 1;
        }
        if ( menuapp->current_media == NULL ) {        
            menuapp->current_media = strdup(entry->url);
            menuapp->paused = FALSE;
            dongle_send_play(menuapp->dongle,entry->url);
        } else {
            next_media_clear(menuapp);
            next_media_push(menuapp,entry->url);       
            dongle_send_message(menuapp->dongle,RDC_STOP);
        }
        program_register_keys(menuapp->program,REGISTER_MEDIA,media_keys,menuapp);
        program_register_ack(menuapp->program,REGISTER_MEDIA,media_ack,menuapp);
        break;
    }   
    return 1;
}

/***********************
 *
 *   Handling of the vdrrecs Menus
 *
 ***********************/
static int menuapp_vdrrecs(menu_t *menu,void *imenuapp, int sel)
{
    menuapp_t      *menuapp = (menuapp_t *)imenuapp;
    menu_t         *child;
    dirlist_t      *dirlist;
    filelist_t     *list = NULL;
    int             num  = 0;
    char            buf[1024];
    char            buf2[1024];
    cRecordings     Recordings;

    /* Define a menu */
    dirlist = new_dirlist(menuapp);
    child = new_menu(menuapp->dongle,menuapp->render,menuapp->program,menu,dirlist,0);
    menu_set_param_clearup(child,delete_dirlist);

    menu_set_title(child,tr("Video Recordings"));   
    menu_set_dialcode_callback(child, dial_entry);

    /* Load recordings up from the VDR core */
    bool recordings = Recordings.Load();

    /* Thanks Sascha! - code taken from streamdev-server */
    if ( recordings ) {
        cRecording *recording = Recordings.First();
        while ( recording ) {
            
            snprintf(buf,sizeof(buf),"%s",recording->Title(' ',true));
            snprintf(buf2,sizeof(buf2),"%s%s",VDRRECURL,buf);
            
            list = filelist_add(list,&num,buf,buf2,NULL,FILE_URL);
            menu_add(child,buf,process_media);
          
            recording = Recordings.Next(recording);
        }
    }
   
    /* Now we'll set up dirlist */
    dirlist->title = NULL;    /* We don't need it since there's no subdirs */
    dirlist->path  = NULL;    /* We don't need it sinec there's no subdirs */
    dirlist->filelist_num = num;
    dirlist->filelist = list;
   
    menu_set_colour_actions(child,tr("Back"),NULL,NULL,NULL,vdrrecs_colour_keys);

    menu_display(child);
    return 1;  
}


static int vdrrecs_colour_keys(menu_t *menu, void *imenuapp, int key, int sel)
{
    switch ( key ) {
    case keyRed:
        return 0;   
    default:
        break;
    }

    return 1;     /* Carry on displaying this menu */
}


/* Deal with Chan+/Chan- */
int  media_keys_vdr(menuapp_t *menuapp,int code)
{
    int            streamtype = dongle_get_type(menuapp->dongle);
    
    switch ( code ) {
    case keyChanUp:
    case keyChanDn:
        if (  ( (streamtype & (MEDIA_LIVE | MEDIA_MPEG) ) == (MEDIA_LIVE | MEDIA_MPEG) ) ) {
            if ( menuapp->current_media && strncmp(menuapp->current_media,VDRURL,strlen(VDRURL)) == 0  ) {
                char           buf[2048];
                const char    *newchannel = ChannelSkip(menuapp->current_media + strlen(VDRURL),
                                                        code == keyChanUp ? 1 : -1 );
                if ( newchannel ) {
                    if ( menuapp->current_media ) {        
                        free(menuapp->current_media);
		    }
                    snprintf(buf,sizeof(buf),"%s%s",VDRURL,newchannel);
                    menuapp->current_media = strdup(buf);
                    menuapp->paused = FALSE;
                    dongle_send_play(menuapp->dongle,buf);
                }
            }
            return 1;  /* We've done all we can.. */
        } else {
            return 0;   /* Carry on with menu */
        }
    }

    return -1;         /* Carry on with normal key functions */
}


/* Scan a file in the format used by the mp3 plugin */
static filelist_t *plugin_file_scan(char *filename, int *scanned)
{
    char         buf[FILENAME_MAX+1];
    filelist_t  *list = NULL;
    int          count = 0;
    char        *ptr,*endptr;
    struct stat  sb;
    FILE        *fp;
    int          flags;

    *scanned = 0;


    if ( stat(filename,&sb) < 0 || ( fp = fopen(filename,"r") ) == NULL ) {
        return NULL;
    }
  
    while ( !feof(fp) && fgets(buf,sizeof(buf),fp) ) {
        if ( (ptr = strchr(buf,';') ) == NULL ) {
            continue;
        }
        *ptr = 0;
        ptr++;

        if ( (endptr = strchr(ptr,';') ) == NULL && 
             (endptr = strchr(ptr,'\n') ) == NULL &&
             (endptr = strchr(ptr,'\r') ) == NULL ) {
            continue;
        }
        *endptr = 0;

        if ( stat(buf,&sb) < 0 ) {
            continue;
        }

        if ( S_ISDIR(sb.st_mode) ) {
            flags = FILE_DIR;
        } else {
            flags = 0;
        }

        list = filelist_add(list,&count,ptr,buf,NULL,flags);       
    }

    *scanned = count;
    return list;    
}
